@@ -2,7 +2,7 @@ from enum import Enum
 from functools import lru_cache
 from typing import Any, Callable, Dict, Mapping, Optional, Tuple, Type, TypeVar
 
-from attr import fields, has, resolve_types
+from attr import resolve_types, has as attrs_has
 
 from ._compat import (
     get_origin,
@@ -14,6 +14,9 @@ from ._compat import (
     is_sequence,
     is_tuple,
     is_union_type,
+    is_annotated,
+    has,
+    fields,
 )
 from .disambiguators import create_uniq_field_dis_func
 from .dispatch import MultiStrategyDispatch
@@ -21,6 +24,7 @@ from .gen import (
     AttributeOverride,
     make_dict_structure_fn,
     make_dict_unstructure_fn,
+    make_iterable_unstructure_fn,
 )
 
 NoneType = type(None)
@@ -54,8 +58,10 @@ class Converter(object):
     )
 
     def __init__(
-        self, dict_factory=dict, unstruct_strat=UnstructureStrategy.AS_DICT
-    ):
+        self,
+        dict_factory: Callable[[], Any] = dict,
+        unstruct_strat: UnstructureStrategy = UnstructureStrategy.AS_DICT,
+    ) -> None:
         unstruct_strat = UnstructureStrategy(unstruct_strat)
 
         # Create a per-instance cache.
@@ -147,18 +153,17 @@ class Converter(object):
         The converter function should take an instance of the class and return
         its Python equivalent.
         """
-        if has(cls):
+        if attrs_has(cls):
             resolve_types(cls)
         if is_union_type(cls):
             self._unstructure_func.register_func_list(
-                [(lambda t: t is cls, func)]
+                [(lambda t: t == cls, func)]
             )
         else:
-            self._unstructure_func.clear_direct()
             self._unstructure_func.register_cls_list([(cls, func)])
 
     def register_unstructure_hook_func(
-        self, check_func, func: Callable[[T], Any]
+        self, check_func: Callable[[Any], bool], func: Callable[[T], Any]
     ):
         """Register a class-to-primitive converter function for a class, using
         a function to check if it's a match.
@@ -177,12 +182,11 @@ class Converter(object):
         and return the instance of the class. The type may seem redundant, but
         is sometimes needed (for example, when dealing with generic classes).
         """
-        if has(cl):
+        if attrs_has(cl):
             resolve_types(cl)
         if is_union_type(cl):
             self._union_struct_registry[cl] = func
         else:
-            self._structure_func.clear_direct()
             self._structure_func.register_cls_list([(cl, func)])
 
     def register_structure_hook_func(
@@ -203,7 +207,7 @@ class Converter(object):
     # Classes to Python primitives.
     def unstructure_attrs_asdict(self, obj) -> Dict[str, Any]:
         """Our version of `attrs.asdict`, so we can call back to us."""
-        attrs = obj.__class__.__attrs_attrs__
+        attrs = fields(obj.__class__)
         dispatch = self._unstructure_func.dispatch
         rv = self._dict_factory()
         for a in attrs:
@@ -214,7 +218,7 @@ class Converter(object):
 
     def unstructure_attrs_astuple(self, obj) -> Tuple[Any, ...]:
         """Our version of `attrs.astuple`, so we can call back to us."""
-        attrs = obj.__class__.__attrs_attrs__
+        attrs = fields(obj.__class__)
         dispatch = self._unstructure_func.dispatch
         res = list()
         for a in attrs:
@@ -298,7 +302,7 @@ class Converter(object):
     ) -> T:
         """Load an attrs class from a sequence (tuple)."""
         conv_obj = []  # A list of converter parameters.
-        for a, value in zip(cl.__attrs_attrs__, obj):  # type: ignore
+        for a, value in zip(fields(cl), obj):  # type: ignore
             # We detect the type by the metadata.
             converted = self._structure_attr_from_tuple(a, a.name, value)
             conv_obj.append(converted)
@@ -321,7 +325,7 @@ class Converter(object):
 
         conv_obj = {}  # Start with a fresh dict, to ignore extra keys.
         dispatch = self._structure_func.dispatch
-        for a in cl.__attrs_attrs__:  # type: ignore
+        for a in fields(cl):  # type: ignore
             # We detect the type by metadata.
             type_ = a.type
             name = a.name
@@ -455,9 +459,7 @@ class Converter(object):
                 e for e in union_types if e is not NoneType  # type: ignore
             )
 
-        if not all(
-            hasattr(get_origin(e) or e, "__attrs_attrs__") for e in union_types
-        ):
+        if not all(has(get_origin(e) or e) for e in union_types):
             raise ValueError(
                 "Only unions of attr classes supported "
                 "currently. Register a loads hook manually."
@@ -472,9 +474,9 @@ class GenConverter(Converter):
 
     def __init__(
         self,
-        dict_factory=dict,
-        unstruct_strat=UnstructureStrategy.AS_DICT,
-        omit_if_default=False,
+        dict_factory: Callable[[], Any] = dict,
+        unstruct_strat: UnstructureStrategy = UnstructureStrategy.AS_DICT,
+        omit_if_default: bool = False,
         type_overrides: Mapping[Type, AttributeOverride] = {},
     ):
         super().__init__(
@@ -500,6 +502,44 @@ class GenConverter(Converter):
                 ]
             )
 
+        self._unstructure_func.register_func_list(
+            [
+                (is_annotated, self.gen_unstructure_annotated, True),
+                (
+                    is_sequence,
+                    self.gen_unstructure_iterable,
+                    True,
+                ),
+                (
+                    is_mutable_set,
+                    lambda cl: self.gen_unstructure_iterable(
+                        cl, unstructure_to=set
+                    ),
+                    True,
+                ),
+                (
+                    is_frozenset,
+                    lambda cl: self.gen_unstructure_iterable(
+                        cl, unstructure_to=frozenset
+                    ),
+                    True,
+                ),
+            ]
+        )
+        self._structure_func.register_func_list(
+            [(is_annotated, self.gen_structure_annotated, True)]
+        )
+
+    def gen_unstructure_annotated(self, type):
+        origin = type.__origin__
+        h = self._unstructure_func.dispatch(origin)
+        return h
+
+    def gen_structure_annotated(self, type):
+        origin = type.__origin__
+        h = self._structure_func.dispatch(origin)
+        return h
+
     def gen_unstructure_attrs_fromdict(self, cl: Type[T]) -> Dict[str, Any]:
         attribs = fields(cl)
         if any(isinstance(a.type, str) for a in attribs):
@@ -531,3 +571,10 @@ class GenConverter(Converter):
         self._structure_func.register_cls_list([(cl, h)], direct=True)
         # only direct dispatch so that subclasses get separately generated
         return h
+
+    def gen_unstructure_iterable(self, cl: Any, unstructure_to=list):
+        h = make_iterable_unstructure_fn(
+            cl, self, unstructure_to=unstructure_to
+        )
+        self._unstructure_func.register_cls_list([(cl, h)], direct=True)
+        return h
